#include <stdio.h>
#include <stdlib.h>

#include "operations.h"
#include "tap.h"

// NO LONGER USED
char* _operations_compute_extract_block_filename(int index) {
	const int MAX_FILE_NAME = 8128;
	char* blockFileName = (char*)malloc((MAX_FILE_NAME + 1) * sizeof(char));
	if (!blockFileName) {
		printf("Error: could not allocate memory for file name of block to extract\n");
		return NULL;
	}

	sprintf_s(blockFileName, MAX_FILE_NAME, "%s_block%d\n",
		tap_get_filename(),
		index);
	return blockFileName;
}

void operations_extract_block(char* outputFileName, int index) {
	if (!tap_is_file_loaded()) {
		printf("Error: a TAP file must be specified before a block can be extracted\n");
		return;
	}

	unsigned short blocksCount = tap_get_block_count();
	if (index < 0 || index >= blocksCount) {
		printf("Error: index of block to extract is out of range 0..%d\n", blocksCount);
		return;
	}

	// open file to which we're extracting the block
	FILE* fp;
	errno_t result = fopen_s(&fp, outputFileName, "wb");
	if (result) {
		printf("Error: could not create extracted block file %s\n", outputFileName);
		return;
	}
	
	// write the block
	struct tapBlock* block = tap_get_block(index);
	size_t blocksWritten = fwrite(block->tapBlockBytes, block->tapBlockSize, 1, fp);
	if (blocksWritten != 1) {
		printf("Error: could not write block at index %d, of size 0x%04X, to file %s\n",
			index,
			block->tapBlockSize,
			outputFileName);
		return;
	}
	fclose(fp);

	printf("Extracted block %d of size 0x%04X to file %s\n",
		index,
		block->tapBlockSize,
		outputFileName);
}

//struct tapBlock** _blockPointers;
//
//void _operations_populate_block_pointers() {
//	_blockPointers = (struct tapBlock**)malloc(tap_get_block_count() * sizeof(struct tapBlock*));
//
//}

void operations_insert(char* insertFileName, char* outputFileName, int insertIndex) {
	// open output file
	FILE* outputFp;
	errno_t result = fopen_s(&outputFp, outputFileName, "wb");
	if (result) {
		printf("Error: could not create output file %s\n", outputFileName);
		return;
	}

	// open file to insert
	FILE* fileToInsertFp;
	result = fopen_s(&fileToInsertFp, insertFileName, "rb");
	if (result) {
		printf("Error: could not open file to insert %s\n", outputFileName);
		fclose(outputFp);
		return;
	}

	// write original TAP file blocks one by one, also writing the data in the
	// file to be inserted - when the appropriate slot is reached

	for (int slot = 0; slot < tap_get_block_count(); slot++) {
		// if we're inserting in this slot, then first write the data to be inserted
		if (insertIndex == slot) {
			char aByte;
			while (fread(&aByte, 1, 1, fileToInsertFp) > 0) {
				if (fwrite(&aByte, 1, 1, outputFp) != 1) {
					printf("Error: could not write byte into output file\n");
					fclose(fileToInsertFp);
					fclose(outputFp);
					return;
				}
			}
		}

		// write the block
		struct tapBlock* block = tap_get_block(slot);
		if (fwrite(block->tapBlockBytes, block->tapBlockSize, 1, outputFp) != 1) {
			printf("Error: could not write block at index %d\n",
				slot);
			fclose(fileToInsertFp);
			fclose(outputFp);
			return;
		}
	}

	// if we're appending, then the specified insert index will be past the last index
	// of the original TAP file
	// this means we haven't yet written it in the loop above, and must write it now
	if (insertIndex >= tap_get_block_count()) {
		char aByte;
		while (fread(&aByte, 1, 1, fileToInsertFp) > 0) {
			if (fwrite(&aByte, 1, 1, outputFp) != 1) {
				printf("Error: could not write byte into output file\n");
				fclose(fileToInsertFp);
				fclose(outputFp);
				return;
			}
		}
	}

	fclose(fileToInsertFp);
	fclose(outputFp);

	printf("Inserted file %s into slot %d of file %s - wrote result as file %s\n",
		insertFileName,
		insertIndex,
		tap_get_filename(),
		outputFileName);
}
